
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */
 
package org.aspectj.tools.ajde.netbeans;

import java.util.*;
import java.io.*;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.*;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleInstall;
import org.openide.util.actions.SystemAction;
import org.openide.explorer.*;
import org.openide.execution.*;
import org.openide.explorer.view.*;
import org.openide.nodes.*;
import org.openide.text.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.*;
import org.netbeans.core.*;
import org.netbeans.core.ExLocalFileSystem;
import org.openide.src.*;
import org.aspectj.ajde.internal.*;
import org.aspectj.ajde.*;
import org.aspectj.util.ConfigFileUtil;

/**
 * @author	Mik Kersten
 */
public class BuildConfigurationManager extends LstBuildConfigManager {
	
    public String getDefaultConfigFile() {        
        List configFiles = Ajde.getDefault().getProjectProperties().getProjectSourceFiles();

        String projectName = Ajde.getDefault().getProjectProperties().getProjectName();
        String filePath = Ajde.getDefault().getProjectProperties().getOutputPath() + '/' +
            "default-config.lst";
        //LstBuildConfigFileUpdater.writeConfigFile(filePath, configFiles);
        return filePath;
//        List configs = getConfigFiles();
//        if (configs != null && configs.size() > 0) {
//            return (String)configs.get(0);
//        } else {
//            return null;
//        }
//           
    }
  
    public List getConfigFiles() {
        List configs = new ArrayList();
        configs.add(BuildConfigManager.DEFAULT_CONFIG_LABEL);
        Repository repository = org.openide.TopManager.getDefault().getRepository();
        Enumeration filesystems = repository.getFileSystems();
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof ExLocalFileSystem) {
                ExLocalFileSystem exfs = (ExLocalFileSystem)fs;
                if (new File(exfs.getSystemName()).exists()) {
                    configs.addAll(getLstFilesInDir(exfs.getSystemName()));
                }
            }
        }
        if (configs.size() == 0) {
            Ajde.getDefault().getErrorHandler().handleWarning("Could not find any \".lst\" in the project filesystems.");
        }
        return configs;
    }
	  
    /**
     * Retrieves all of the .lst files in a directory, replacing any '\' characters with '/'.
     *
     * @returns an empty List if no files were found
	 */
    private List getLstFilesInDir(String dirPath) { // todo: why not use a FileFilter?
        List configs = new ArrayList();
        File f = new File(dirPath);
        File[] dirContents = f.listFiles();
        for (int j = 0; j < dirContents.length; j++) {
            if (dirContents[j].isDirectory()) {
                configs.addAll(getLstFilesInDir(dirContents[j].getAbsolutePath()));
            } else if (dirContents[j].getName().endsWith(".lst")) {
                configs.add(dirContents[j].getAbsolutePath().replace('\\', '/'));
            }
        }
        return configs;
    } 
    
	public String getCurrConfigFile() {
		if (currConfigFilePath == null) return null;
		if (currConfigFilePath.equals(DEFAULT_CONFIG_LABEL)) {
			return getDefaultConfigFile();
		} else {
			return currConfigFilePath;	
		}
	}
}
