
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.openide.windows.*;
import org.aspectj.ajde.*;
import org.aspectj.asm.*;
import org.aspectj.ajde.ui.swing.*;

public class NbCompilerMessages implements TaskListManager {

    private CompilerMessageWindow cmw = new CompilerMessageWindow();
    private TopComponent comp = new TopComponent();
	private int x = 0;
	private int y = 0;

    public NbCompilerMessages() {
        comp.setLayout(new BorderLayout());
        comp.add(cmw, BorderLayout.CENTER);
        comp.setPreferredSize(new Dimension (600, 200));
        comp.setName("AspectJ Build Msgs");  
        //comp.open();
        x = org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getX() +
                org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getHeight();
        y = org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getY() +
                org.openide.TopManager.getDefault().getWindowManager().getMainWindow().getWidth();
    }
    
    public void addSourcelineTask(String message, SourceLocation sourceLocation, StructureMessage.Kind kind) {
	    cmw.addMessage(message, kind, sourceLocation);
        if (!comp.isOpened()) comp.open();
        comp.setLocation(x, y);
        comp.requestFocus();
    }

	public void addProjectTask(String message, StructureMessage.Kind kind) {
    	cmw.addMessage(message, kind, null);	
    	if (!comp.isOpened()) comp.open();
    	comp.setLocation(x, y);
    	comp.requestFocus();
    }
  
    public void clearTasks() {
        //comp.close();
        cmw.clearMessages();  
    }

    class CompilerMessageWindow extends JPanel {
        JScrollPane jScrollPane1 = new JScrollPane();
        JScrollPane jScrollPane2 = new JScrollPane();
        JList list = new JList();
        DefaultListModel listModel = new DefaultListModel();
        BorderLayout borderLayout1 = new BorderLayout();

        public CompilerMessageWindow() {
            try {
                jbInit();
            }
            catch(Exception e) {
                e.printStackTrace();
            }
            list.setModel(listModel);

             MouseListener mouseListener = new MouseAdapter() {
                 public void mouseClicked(MouseEvent e) {
                     if (e.getClickCount() == 2) {
                         int index = list.locationToIndex(e.getPoint());
                         CompilerMessage cm = (CompilerMessage)listModel.getElementAt(index);
                         Ajde.getDefault().getEditorManager().showSourceLine(
                         	cm.sourceLocation.getSourceFilePath(), 
                       		cm.sourceLocation.getLineNumber(), 
                         	true);
                      }
                 }
             };
             list.addMouseListener(mouseListener);
			list.setCellRenderer(new CompilerMessagesCellRenderer());
        }

        public void addMessage(String message, StructureMessage.Kind kind, SourceLocation sourceLocation) {
	        if (sourceLocation != null) {
		        listModel.addElement(new CompilerMessage(message, sourceLocation, kind));
	        } else {
	 			listModel.addElement(new CompilerMessage(message, kind));
	        }
        }

        public void clearMessages() {
            listModel.clear();
        }

        private void jbInit() throws Exception {
            this.setLayout(borderLayout1);
            this.add(jScrollPane1, BorderLayout.CENTER);
            jScrollPane1.getViewport().add(jScrollPane2, null);
            jScrollPane2.getViewport().add(list, null);
        }
    }
}
