

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s): 
 */

package org.aspectj.tools.ajde.netbeans;

import java.io.*;
import java.util.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.text.StyledDocument;
import org.openide.TopManager;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.*;
import org.openide.text.Line;
import org.openide.text.Annotation;
import org.openide.text.Annotatable;
import org.aspectj.asm.*;
import org.aspectj.ajde.*;

public class NbEditorAdapter implements EditorAdapter {

    public String getCurrFile() {
        return null;
    }

    public String getCurrSourceFilePath() {
        return "";
    }

    public void showSourceLine(int lineNumber, boolean highlight) {
        showSourceLine(getCurrFile(), lineNumber, true); 
    }

	public void showSourceLine(SourceLocation sourceLocation, boolean highlight) {
		showSourceLine(
			sourceLocation.getSourceFilePath(),
			sourceLocation.getLineNumber(),
			highlight);		
	}

    public void showSourceLine(String filePath, int lineNumber, boolean highlight) {
        //final Annotation ann = new LineMarker();
        
        if (filePath == "") {
            return;
        }
        try {
            filePath = filePath.replace('\\', '/');
            String currFile = filePath.substring(filePath.lastIndexOf('/')+1, filePath.lastIndexOf('.'));
            String packageName = "";

            Vector filesystems = getFilesystemPaths();
            Iterator it = filesystems.iterator();
            while (it.hasNext()) {
                String filesystem = (String)it.next();
                filesystem = filesystem.replace('\\', '/');
                if (filePath.startsWith(filesystem)) {
                    try {
                        packageName = filePath.substring(filesystem.length() + 1, filePath.lastIndexOf('/')).replace('/', '.');
                    } catch (StringIndexOutOfBoundsException sioobe) { }
                }
            }

            FileObject fo = null;
            if (filePath.endsWith(".aj")) {
                fo = TopManager.getDefault().getRepository().find(packageName, currFile, "aj");
            } else {
                fo = TopManager.getDefault().getRepository().find(packageName, currFile, "java");
            }

            DataObject dataObject = org.openide.TopManager.getDefault().getLoaderPool().findDataObject(fo);
            EditorCookie ec = (EditorCookie)(dataObject.getCookie(EditorCookie.class));
            ec.open();
            
            int lineToSeekTo;
            if (lineNumber < 0) {
                lineToSeekTo = 1;
            } else {
                lineToSeekTo = lineNumber - 1;
            }

			try {
	            if (lineNumber != 1) {
	                ec.getLineSet().getCurrent(lineToSeekTo).markCurrentLine();  
				}  
	            ec.getLineSet().getCurrent(lineToSeekTo).show(Line.SHOW_GOTO);
			} catch (NullPointerException npe) {
				Ajde.getDefault().logEvent("Could not seek to line: " + npe.getMessage());	
			}
                //ec.getLineSet().getCurrent(lineToSeekTo).add
// TODO: Figure this out...pjs                
// markCurrentLine() is deprecated and replaced with Annotation.attach(line). 
// However, one must detach the Annotation or the line remains marked "FOREVER"...    

//                final Line line = ec.getLineSet().getCurrent(lineToSeekTo);//.markCurrentLine();
//                ann.attach(line);
                
/*                line.addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent ev) {
                        String type = ev.getPropertyName();
 * maybe PROP_ANNOTATION_COUNT can be used to trigger the detach by saving a static count and then comparing...
                        if (type == null || type == Annotatable.PROP_TEXT || type == Annotatable.PROP_ANNOTATION_COUNT) {
                            // User edited the line, assume error should be cleared.
                            ann.detach();
                            line.removePropertyChangeListener(this);
                        }
                    }
                });

 */
        } catch (Exception e) {
            Ajde.getDefault().getErrorHandler().handleError("Could not seek to line in file: " + filePath + ".", e);
            e.printStackTrace();
        }
    }

	/**
	 * Not implemented.
	 */
	public void showSourcelineAnnotation(String filePath, int lineNumber, List items) { }

    /**
     * Not implemented.
     */
    public void pasteToCaretPos(String text) { }

    /**
     * Not implemented.
     */
    public void showSourcelineAnnotation(String filePath, int lineNumber, JPopupMenu annotation) { }

    public void saveContents() throws IOException {
    }

    public JPanel getPanel() {
        return null;
    }

    private Vector getFilesystemPaths() {
        Vector filesystemPaths = new Vector();
        Repository repository = TopManager.getDefault().getRepository();
        Enumeration filesystems = repository.getFileSystems();
        File f = null;
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof org.netbeans.core.ExLocalFileSystem) {
                f = new File(fs.getSystemName());
                filesystemPaths.add(f.getAbsolutePath());
            }
        }
        return filesystemPaths;
    }   
}

//import org.openide.text.Annotation;
//
///** 
// *  Supplies an annotation to mark a line in the editor.
// *  @author Phil Sager
// */
//public final class LineMarker extends Annotation {
////    private final String error;
//    
//    public LineMarker() {
//        
//    }
//    
//    public String getAnnotationType() {
//        return "org-aspectj-tools-ajde-netbeans-LineMarker";
//    }
//    
//    public String getShortDescription() {
//        // Localize this with NbBundle:
//        return "LineMarker";
//    }
//}