
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.net.URL;
import javax.swing.*;
import org.openide.TopManager;
import org.aspectj.ajde.ui.*;
import org.aspectj.ajde.ui.swing.IconRegistry;
import org.aspectj.asm.*;

public class NbIconRegistry extends IconRegistry {

    public static final NbIconRegistry INSTANCE = new NbIconRegistry();
    public static final String NB_STRUCTURE_PATH = "org/openide/resources/src/";

	protected final Icon PROJECT = createNBIcon(NB_STRUCTURE_PATH + "localFS.gif");
	protected final Icon PACKAGE = createNBIcon(NB_STRUCTURE_PATH + "defaultFolder.gif");

	protected final Icon METHOD = createNBIcon(NB_STRUCTURE_PATH + "methodPublic.gif");
	protected final Icon METHOD_PUBLIC = createNBIcon(NB_STRUCTURE_PATH + "methodPublic.gif");
	protected final Icon METHOD_PACKAGE = createNBIcon(NB_STRUCTURE_PATH + "methodPackage.gif");
	protected final Icon METHOD_PRIVATE = createNBIcon(NB_STRUCTURE_PATH + "methodPrivate.gif");
	protected final Icon METHOD_PROTECTED = createNBIcon(NB_STRUCTURE_PATH + "methodProtected.gif");
	
	protected final Icon FIELD = createNBIcon(NB_STRUCTURE_PATH + "variablePublic.gif");
	protected final Icon FIELD_PUBLIC = createNBIcon(NB_STRUCTURE_PATH + "variablePublic.gif");
	protected final Icon FIELD_PACKAGE = createNBIcon(NB_STRUCTURE_PATH + "variablePackage.gif");
	protected final Icon FIELD_PRIVATE = createNBIcon(NB_STRUCTURE_PATH + "variablePrivate.gif");
	protected final Icon FIELD_PROTECTED = createNBIcon(NB_STRUCTURE_PATH + "variableProtected.gif");
	
	protected final Icon CLASS = createNBIcon(NB_STRUCTURE_PATH + "class.gif");
	protected final Icon INTERFACE = createNBIcon(NB_STRUCTURE_PATH + "interface.gif");
    
	public Icon getStructureSwingIcon(ProgramElementNode.Kind kind) { 		
		return getStructureSwingIcon(kind, ProgramElementNode.Accessibility.PUBLIC);
	}

	public Icon getStructureSwingIcon(ProgramElementNode.Kind kind, ProgramElementNode.Accessibility accessibility) { 
		if (kind == ProgramElementNode.Kind.METHOD) {
			if (accessibility == ProgramElementNode.Accessibility.PUBLIC) {
				return METHOD_PUBLIC;	
			} else if (accessibility == ProgramElementNode.Accessibility.PACKAGE) {
				return METHOD_PACKAGE;	
			} else if (accessibility == ProgramElementNode.Accessibility.PROTECTED) {
				return METHOD_PROTECTED;	
			} else if (accessibility == ProgramElementNode.Accessibility.PRIVATE) {
				return METHOD_PRIVATE;	
			} else {
				return METHOD;
			}
		} else if (kind == ProgramElementNode.Kind.FIELD) {
			if (accessibility == ProgramElementNode.Accessibility.PUBLIC) {
				return FIELD_PUBLIC;	
			} else if (accessibility == ProgramElementNode.Accessibility.PACKAGE) {
				return FIELD_PACKAGE;	
			} else if (accessibility == ProgramElementNode.Accessibility.PROTECTED) {
				return FIELD_PROTECTED;	
			} else if (accessibility == ProgramElementNode.Accessibility.PRIVATE) {
				return FIELD_PRIVATE;	
			} else {
				return FIELD;
			}
		} else if (kind == ProgramElementNode.Kind.INTERFACE) {
			return INTERFACE;
		} else if (kind == ProgramElementNode.Kind.CLASS) {
			return CLASS;
		} else {
			return (Icon)getStructureIcon(kind).getIconResource();
		}
	}

	protected Icon createNBIcon(String path) {
        URL imageURL = TopManager.getDefault().currentClassLoader().getResource(path);
        try {
            return new ImageIcon(imageURL);
        } catch (Exception e) {
            return null;
        }	
	}

	protected AbstractIcon createIcon(String path) {
		return new AbstractIcon(createNBIcon(path));	
	}

    protected Icon makeIcon(String iconPath) {    	
        URL imageURL = TopManager.getDefault().currentClassLoader().getResource(RESOURCE_PATH + iconPath);
        try {
            return new ImageIcon(imageURL);
        } catch (Exception e) {
            return null;
        }
    }
}

