
/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the IDE support for the AspectJ(tm)
 * programming language; see http://aspectj.org
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AspectJ.
 *
 * The Initial Developer of the Original Code is Xerox Corporation. Portions
 * created by Xerox Corporation are Copyright (C) 1999-2002 Xerox Corporation.
 * All Rights Reserved.
 *
 * Contributor(s):
 */

package org.aspectj.tools.ajde.netbeans;

import java.util.*;
import java.io.*;
import javax.swing.*;
import org.openide.*;

import org.aspectj.tools.ide.*;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.*;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleInstall;
import org.openide.util.actions.SystemAction;
//import org.openidex.util.Utilities2;
import org.openide.explorer.*;
import org.openide.execution.*;
import org.openide.explorer.view.*;
import org.openide.nodes.*;
import org.openide.text.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.*;
import org.netbeans.core.*;
import org.openide.src.*;

import org.openide.compiler.*;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CallableSystemAction;
import org.openide.windows.*;
import org.openide.loaders.DataFolder;
import org.openide.modules.ModuleInstall;
import org.openide.util.actions.SystemAction;
import org.openide.explorer.*;
import org.openide.execution.*;
import org.openide.explorer.view.*;
import org.openide.options.*;
import org.openide.nodes.*;
import org.openide.text.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.cookies.*;
import org.netbeans.core.*;
import org.openide.filesystems.*;
import org.aspectj.ajde.*;
import org.aspectj.ajde.ui.*;
import org.aspectj.asm.*;

public class NbProjectProperties implements ProjectPropertiesAdapter {

	private UserPreferencesAdapter preferencesAdapter = null;
	
	public NbProjectProperties(UserPreferencesAdapter preferencesAdapter) {
		this.preferencesAdapter = preferencesAdapter;	
	}

    public String getDefaultBuildConfigFile() {        
        List configFiles = Ajde.getDefault().getProjectProperties().getProjectSourceFiles();

        String projectName = Ajde.getDefault().getProjectProperties().getProjectName();
        String filePath = Ajde.getDefault().getProjectProperties().getOutputPath() + '/' +
            "default-config.lst";
            
        //LstBuildConfigFileUpdater.writeConfigFile(filePath, configFiles);
        
        return filePath;
//        List configs = getConfigFiles();
//        if (configs != null && configs.size() > 0) {
//            return (String)configs.get(0);
//        } else {
//            return null;
//        }
//           
    }
  
    public List getBuildConfigFiles() {
        List configs = new ArrayList();
        configs.add(BuildConfigManager.DEFAULT_CONFIG_LABEL);
        Repository repository = org.openide.TopManager.getDefault().getRepository();
        Enumeration filesystems = repository.getFileSystems();
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof ExLocalFileSystem) {
                ExLocalFileSystem exfs = (ExLocalFileSystem)fs;
                if (new File(exfs.getSystemName()).exists()) {
                    configs.addAll(getLstFilesInDir(exfs.getSystemName()));
                }
            }
        }
        if (configs.size() == 0) {
            Ajde.getDefault().getErrorHandler().handleWarning("Could not find any \".lst\" in the project filesystems.");
        }
        return configs;
    }
	  
	 public String getLastActiveBuildConfigFile() {
	 	return null;	
	 }
	  
    /**
     * Retrieves all of the .lst files in a directory, replacing any '\' characters with '/'.
     *
     * @returns an empty List if no files were found
	 */
    private List getLstFilesInDir(String dirPath) { // todo: why not use a FileFilter?
        List configs = new ArrayList();
        File f = new File(dirPath);
        File[] dirContents = f.listFiles();
        for (int j = 0; j < dirContents.length; j++) {
            if (dirContents[j].isDirectory()) {
                configs.addAll(getLstFilesInDir(dirContents[j].getAbsolutePath()));
            } else if (dirContents[j].getName().endsWith(".lst")) {
                configs.add(dirContents[j].getAbsolutePath().replace('\\', '/'));
            }
        }
        return configs;
    } 

    public boolean isGlobalMode() {
        return true;
    }

    public String getAjcWorkingDir() {
        return getRootProjectDir();
    }
    
    public String getOutputPath() {
        Vector filesystemPaths = new Vector();
        Enumeration filesystems = TopManager.getDefault().getRepository().getFileSystems();

		String path = "";
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof org.netbeans.core.ExLocalFileSystem) {
            	org.openide.filesystems.LocalFileSystem exfs = (org.openide.filesystems.LocalFileSystem)fs;
            	path = exfs.getRootDirectory().getAbsolutePath();
            	return path;
            }
        }
        Ajde.getDefault().getErrorHandler().handleWarning("Build output path could not be determined, using \".\"");
        return ".";
    }

	public String getClassToExecute() {
		return null;	
	}

	public String getExecutionArgs() {
		return null;	
	}

	public String getVmArgs() {
		return null;		
	}

	public String getProjectName() {
		return null;	
	}

    public String getRootProjectDir() {
        Repository repository = org.openide.TopManager.getDefault().getRepository();
        Enumeration filesystems = repository.getFileSystems();
        File f = null;
        while (filesystems.hasMoreElements()) {
              FileSystem fs = (FileSystem)filesystems.nextElement();
              if (fs instanceof org.netbeans.core.ExLocalFileSystem) {
                    f = new File(fs.getSystemName());
                    if (f.getPath().endsWith("ajworkingdir")) {
                        return f.getAbsolutePath();
                    }
              }
        }
        return null;
    }

    public String getProjectSourcePath() {
        Vector filesystemPaths = new Vector();
        Enumeration filesystems = org.openide.TopManager.getDefault().getRepository().getFileSystems();
        File f = null;
        String paths = "";
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof org.openide.filesystems.LocalFileSystem &&
                !fs.getSystemName().endsWith("ajworkingdir")) {
                f = new File(fs.getSystemName());
                paths += f.getAbsolutePath() + ";";
            }
        }
        return paths;
    }

	/**
	 * @todo	fix the ugly loop
	 */
    public List getProjectSourceFiles() {
        List projectFiles = new ArrayList();

        Enumeration enum = org.openide.TopManager.getDefault().getRepository().getFileSystems();
        while (enum.hasMoreElements()) {
            FileSystem fileSystem = (FileSystem)enum.nextElement();
            if (!(fileSystem instanceof ExLocalFileSystem)) continue;
            ExLocalFileSystem exLocalFileSystem = (ExLocalFileSystem)fileSystem;
            File rootFile = exLocalFileSystem.getRootDirectory();
            List dirs = new Vector();
            List files = new Vector();
            dirs.add(rootFile);
            Vector classElements = new Vector();
            while (dirs.size() > 0) {
                File dir = (File)dirs.remove(0);
                File[] fileArray = dir.listFiles(new FileFilter() {
                        public boolean accept(File f) {
                            return f.isDirectory() || f.getName().endsWith(".java");
                        }
                    });
                if (fileArray != null) {
	                for (int i = 0; i < fileArray.length; i++) {
	                    File file = fileArray[i];
	                    if (file.isDirectory()) {
	                        dirs.add(file);
	                    } else {
							projectFiles.add(file.getAbsolutePath());
	                    }
	                }
                }
            }
        }
        return projectFiles;

    }

    public String getClasspath() {
        Vector filesystemPaths = new Vector();
        Enumeration filesystems = org.openide.TopManager.getDefault().getRepository().getFileSystems();
        File f = null;
        String jars = "";
        while (filesystems.hasMoreElements()) {
            FileSystem fs = (FileSystem)filesystems.nextElement();
            if (fs instanceof org.openide.filesystems.JarFileSystem) {
                f = new File(fs.getSystemName());
                jars += f.getAbsolutePath() + ";";
            }
        }
        return jars + System.getProperty("netbeans.home") + "/lib/ext/aspectjrt.jar";
    }

    public String getBootClasspath() {
        return null;
    }
    
    
}